/*
 *
 *	Programme d'analyse lexicale pour langage d'assemblage 68705
 *
 *
 * 	Auteur : FAVARD Laurent
 *	Fichier: LEXICALE.C
 *	Pays   : FRANCE
 *	Date   : 18 Mars 1994
 *	Revision: 26 Mars 1994
 *	Version : 1.0
 *	Release : 1.0
 *	Machine : ATARI Falcon030 TOS 4.2
 *	Processeur cible: MC68705 P3S (Famille 6805)
 */

#include <tos.h>
#include <stdio.h>
#include <string.h>

#include "assemble.h"			/* Dfinitions globales */
#include "instlist.h"			/* Jeu d'instructions   */
#include "erreur.h"
			  
/* 
	Les terminaux propres du langage d'assemblages 
*/

static char ListeDirective[8][10]={"EQU","ORG","END","DCB","DSB",
			    "TEXT","DATA",""};

/* 
	Les terminaux simples du langages 
*/

static char Terminal[10]=",#%$_:\0";	

/*
	Variables globales pour faire communiquer l'analyseur lexicale
	avec l'analyseur syntaxique.
*/

Type_ValLex ValLex;	   	/* Valeur lexicale de l'unit lex  */
long	    Lexline=1;		/* ligne courante dans le fichier */

/*
	Divers
*/

static int Affiche=TRUE;		/* True si caractere replac dans 
				   fichier => non raffich */

/*
		Primitives bas niveaux
*/

static char Upcase(char caractere)
{
  char retour;
  
  retour = (caractere >= 'a' && caractere <= 'z') ? caractere-'a'+'A' : caractere;
  return(retour);
}

static int LireCar(void)
/*
	Se charge de lire un caractere
*/
{
  char caractere;
  
  caractere = getc(fichier);
  if(Affiche && ListingScreen)		/* Autoris d'afficher */
   fprintf(stdout,"%c",caractere);
  else					/* non, on n'affiche pas mais */
    Affiche = TRUE;			/* le prochain OUI */
  return(Upcase(caractere));
}

static void UnLire(int caractere,FILE *fichier)
/*
	Se charge de restituer un caractere
*/
{
  ungetc(caractere,fichier);
  Affiche = FALSE;		/* Ne pas le rafficher */
}

static void LireCh(long *valeur)
/*
	Se charge de lire une valeur numrique dcimale
	(Est appel aprs avoir lu un caractre qui a t restitu
	au fichier)
*/
{
  fscanf(fichier,"%ld",valeur);
  fprintf(stdout,"\033D");		/* curseur -1  gauche */
  if(ListingScreen)
    fprintf(stdout,"%ld",*valeur);	/* vers l'cran */
  Affiche = TRUE;			/* rautoris */
}

static int EstChiffre(char caractere)	  
/*
	retourne TRUE si caractere reprsente un chiffre dcimal.
*/
{
  int retour;
  
  retour = (caractere >= '0' && caractere <= '9') ? TRUE : FALSE;
  return(retour);
}

static int EstHexDigit(char caractere)
/*
	retourne TRUE si caractere reprsente un chiffre hexadcimal.
*/
{
  int retour;
  
  retour = (caractere >= '0' && caractere <= '9') ||
  	   (caractere >= 'A' && caractere <= 'F') ? TRUE : FALSE;
  return(retour);
}

static int EstCaractere(char caractere)
/*
	retourne TRUE si caractere reprsente un caractere alphabtique
	tel que: [a-zA-Z]
*/
{
  int min,maj;
  
  min = (caractere >= 'a' && caractere <= 'z') ? TRUE : FALSE;
  maj = (caractere >= 'A' && caractere <= 'Z') ? TRUE : FALSE;
  return( min | maj );
}

static int EstTerminal(char caractere)
/*
	retourne TRUE si caractere reprsente un terminal simple dfini
	dans la liste des terminaux simples 'terminal'.
*/
{
  int indice=0;
  
  while(Terminal[indice]!='\0' && Terminal[indice]!=caractere)
    indice++;
  return((Terminal[indice]!='\0') | (EstCaractere(caractere)));    
}  	

static int EstIDAlpha(char caractere)
/*
	retourne TRUE si caractere reprsente un caractere autoris pour
	l'criture des identificateurs.
*/

{
  if( EstCaractere(caractere) || EstChiffre(caractere) || caractere =='_')
    return(TRUE);
  else
    return(FALSE);
}

static int EstSeparateur(char caractere)
/*
	retourne TRUE si caractere est un sparateur de texte, blanc ou
	tabulation.
*/
{
  int retour;
  
  retour = (caractere == ' ' || caractere == '\t' || caractere == EOF) 
  	    ? TRUE : FALSE;
  return(retour);
}

static int EstNewline(char caractere)
/*
	retourne TRUE si caractere est un code de fin de ligne.
*/
{
  int retour;
  
  retour = (caractere == '\n' || caractere == '\r') ? TRUE : FALSE;
  return(retour);
}

static int IDreconstruir(char *ident)
/*
	reconstruit un identificateur  partir du flot d'entre.
	retourne TRUE si reconnue sans erreur d'criture syntaxique et
	longueur <=  la longueur max. autorise TAILLEID.
*/
{
  char caractere;
  int  indice=0;
  
  caractere = LireCar();
  if(EstCaractere(caractere))			/* Premier caractere */
    ident[indice++] = caractere;		/* doit etre alphabetique */
  else
    return(FALSE);

  caractere = LireCar();			/* (Lettre|Chiffre|_)* */
  while(EstIDAlpha(caractere) && indice < TAILLEID)
  {
    ident[indice++] = caractere;
    caractere = LireCar();
  }
  ident[indice++] = '\0';
  
  if( !EstIDAlpha(caractere) )
  {
    UnLire(caractere,fichier);
    return(TRUE);
  }
  else
    return(FALSE);
}

static int HEXreconstruir(int *numerique)
/*
	reconstruit une valeur numerique  partir du flot d'entre sous 
	forme hexadcimale.
	retourne TRUE si reconnue sans erreur, syntaxe correcte et limit
	 16 bits.
*/
{
  char caractere;
  char chaine[8];
  int  indice=0;
  unsigned char digit;
  
  chaine[indice++]='0';
  chaine[indice++]='x';

  caractere = LireCar();
  digit=0;
  while(EstHexDigit(caractere) && digit<=TAILLE16)
  {
    chaine[indice++] = caractere;
    digit++;
    caractere = LireCar();
  }
  chaine[indice++] = '\0';
  
  if(digit <=TAILLE16)  
  {
    UnLire(caractere,fichier);
    sscanf(chaine,"%X",numerique); 
    return(TRUE);
  }
  else
    return(FALSE);
}

/*
  ----------------------- Primitives plus haut niveaux --------------------
*/

static int EstDirective(char *lexeme)
/*
	Retourne TRUE si le lexeme pass est une directive du langage
*/
{
  int indice=0;
  int retour;
  
  while( strcmp(ListeDirective[indice],"")!=0  && 
         strcmp(ListeDirective[indice],lexeme)!=0 )
    indice++;
  retour = (strcmp(ListeDirective[indice],"")!=0) ? TRUE : FALSE;
  return(retour);
}

static int EstInst(char *lexeme)
/*
	Retourne TRUE si le lexeme pass est une instruction appartenant
	au langage d'assemblage du MOTOROLA 68705.
*/
{
  int indice=0;
  int retour;
  
  while( strcmp(ListeInst[indice],"")!=0 && 
         strcmp(ListeInst[indice],lexeme)!=0 )
    indice++;
  retour = (strcmp(ListeInst[indice],"")!=0) ? TRUE : FALSE;
  return(retour);
}

static int Directive(char *lexeme)
/*
	Retourne le code associ  la directive dans lexeme.
*/
{
  int indice=0;
  
  while( strcmp(ListeDirective[indice],"")!=0  && 
         strcmp(ListeDirective[indice],lexeme)!=0 )
    indice++;
  return(indice+EQU);
}

static int ValTerminal(char caractere)
/*
*/
{
  int indice=0;
  
  while(Terminal[indice]!='\0' && Terminal[indice]!=caractere)
    indice++;
  return(indice+VIRG);    
}  	


/*
  ----------------------- Analyseur lexical lui meme --------------------
*/

int AnalyseLex(void)
{
  char caractere;
  int  correct;
  long numeriquelong;
    
  while(TRUE)
  {
    caractere = LireCar();

    if(caractere == EOF)			/* Fin de fichier */
    {
      return(FDF);
    }       
    else if(EstSeparateur(caractere))		/* separateurs blancs+EOF */
    {
    }
    else if(EstNewline(caractere))		/* code nouvelle ligne */
    {
      Lexline++;
      return(FDL);
    }
    else if(caractere == ';')			/* Commentaires */
    {						/* Ignorer jusqu' fin */
      caractere = LireCar();			/* de ligne */
      while(!EstNewline(caractere) && caractere!=EOF)
        caractere = LireCar();
      Lexline++;				/* incrmente compteur */
      if(caractere == EOF)
        return(FDF);
      else
        return(FDL);
    }
    else if( EstChiffre(caractere) )		/* Chiffre dcimal */
    {
      UnLire(caractere,fichier);
      LireCh(&numeriquelong);
      if(numeriquelong > (unsigned int)0xFFFF)	/* Limit  16 bits ? */
      {
        LEXerreur(ERR_DEC);
        return(LEXERREUR);
      }
      else
      {
        ValLex.numerique = (int)numeriquelong;
        return(NB);
      }
    }
    else if( caractere == '$' )			/* Chiffre hexadcimal */
    {
      correct = HEXreconstruir(&ValLex.numerique);
      if(correct)
        return(NB);
      else
      {
        LEXerreur(ERR_HEX);
        return(LEXERREUR);
      }
    }
    else if( EstCaractere(caractere) )		/* Identificateur  */
    {
      UnLire(caractere,fichier);
      correct = IDreconstruir(ValLex.lexeme);	/* Reconstruit l'ID */
      if(correct)				
      {
        if(EstDirective(ValLex.lexeme))		/* directive langage */
          return(Directive(ValLex.lexeme));
        else if(EstInst(ValLex.lexeme))		/* instruction machine */
          return(INST);
        else					/* indentificateur */
          return(ID);    
      }
      else					
      {
        LEXerreur(ERR_ID);
        return(LEXERREUR);
      }
    }
    else if( EstTerminal(caractere) )		/* terminals du langage */
    {
      ValLex.lexeme[0] = caractere;
      ValLex.lexeme[1] = '\0';			/* Pour permettre un '%s' */
      return(ValTerminal(caractere));
    }
    else 					/* lexeme inconnue */
    {
      LEXerreur(ERR_UNKNOW);
      return(LEXERREUR);
    }
  }
}