/*
 * ----------------------------------------------------------------------------------------------
 *				GESTION DES SPECIFICTES DU MC 68HC11.
 *
 *				Jeux d'instructions du HC11 (codeop).
 *				
 *				
 * 	Auteur 	: 	FAVARD Laurent
 *	Fichier	: 	MC68HC11.H
 *	Pays   	: 	FRANCE
 *	Date   	: 	12 October 1996
 *	Revision: 	17 October 1996
 *	Version : 	1.0
 *	Release : 	1.0
 *	Machine : 	ATARI Falcon030 TOS 4.2
 *
 * ----------------------------------------------------------------------------------------------
 */

#ifndef __MC68HC11__
#define __MC68HC11__	__MC68HC11__


#define	MAX_INST	200		/* Number of instructions defined */

/*
 * ----------------------------------------------------------------------------------------------
 *			Definition du type d'une entree dans une table de codes
 * ----------------------------------------------------------------------------------------------
 */
 
#ifndef	__TYPE_TABLE__	
#define	__TYPE_TABLE__	__TYPE_TABLE__

typedef struct {
		 char Inst[LEN_INSTRUCTION];			/* Mnemonique Instruction */
		 char ModeAdressage[LEN_ADRESSAGE];		/* Addressing Mode */
		 int  CodeMachine;				/* Machine Code */
	       } Type_CodeMachine;

#endif

/*
 * ----------------------------------------------------------------------------------------------
 *			prototypes des procedures dediees
 * ----------------------------------------------------------------------------------------------
 */

extern	int HC11btb(char *Instruction, unsigned long AdresseCible, unsigned long CompteurAssemblage);

/*
 * ----------------------------------------------------------------------------------------------
 *			Tables des codes machines du MC 68HC11
 * ----------------------------------------------------------------------------------------------
 */

#define	MAX_INSTHC11	500			/* Number of instructions defined */

#ifndef	__TABLEHC11__
#define	__TABLEHC11__	__TABLEHC11__

static	Type_CodeMachine Table68HC11[MAX_INSTHC11]	=	{

	       			    {"ABA",	"INHERANT",		0x001B},
	       			    {"ABX",	"INHERANT",		0x003A},
				    {"ABY",	"INHERANT",		0x183A},

				    {"ADCA",	"IMMEDIAT8",		0x0089},
				    {"ADCA",	"DIRECT",		0x0099},
				    {"ADCA",	"EXTEND",		0x00B9},
				    {"ADCA",	"INDEXED_8_X",		0x00A9},
				    {"ADCA",	"INDEXED_8_Y",		0x18A9},

				    {"ADCB",	"IMMEDIAT8",		0x00C9},
				    {"ADCB",	"DIRECT",		0x00D9},
				    {"ADCB",	"EXTEND",		0x00F9},
				    {"ADCB",	"INDEXED_8_X",		0x00E9},
				    {"ADCB",	"INDEXED_8_Y",		0x18E9},

				    {"ADDA",	"IMMEDIAT8",		0x008B},
				    {"ADDA",	"DIRECT",		0x009B},
				    {"ADDA",	"EXTEND",		0x00BB},
				    {"ADDA",	"INDEXED_8_X",		0x00AB},
				    {"ADDA",	"INDEXED_8_Y",		0x18AB},


				    {"ADDB",	"IMMEDIAT8",		0x00CB},
				    {"ADDB",	"DIRECT",		0x00DB},
				    {"ADDB",	"EXTEND",		0x00FB},
				    {"ADDB",	"INDEXED_8_X",		0x00EB},
				    {"ADDB",	"INDEXED_8_Y",		0x18EB},

				    {"ADDD",	"IMMEDIAT16",		0x00C3},
				    {"ADDD",	"DIRECT",		0x00D3},
				    {"ADDD",	"EXTEND",		0x00F3},
				    {"ADDD",	"INDEXED_8_X",		0x00E3},
				    {"ADDD",	"INDEXED_8_Y",		0x18E3},

				    {"ANDA",	"IMMEDIAT8",		0x0084},
				    {"ANDA",	"DIRECT",		0x0094},
				    {"ANDA",	"EXTEND",		0x00B4},
				    {"ANDA",	"INDEXED_8_X",		0x00A4},
				    {"ANDA",	"INDEXED_8_Y",		0x18A4},

				    {"ANDB",	"IMMEDIAT8",		0x00C4},
				    {"ANDB",	"DIRECT",		0x00D4},
				    {"ANDB",	"EXTEND",		0x00F4},
				    {"ANDB",	"INDEXED_8_X",		0x00E4},
				    {"ANDB",	"INDEXED_8_Y",		0x18E4},

				    {"ASL",	"EXTEND",		0x0078},
				    {"ASL",	"INDEXED_8_X",		0x0068},
				    {"ASL",	"INDEXED_8_Y",		0x1868},
				    {"ASLA",	"INHERANT",		0x0048},
				    {"ASLB",	"INHERANT",		0x0058},
				    {"ASLD",	"INHERANT",		0x0005},

				    {"ASR",	"EXTEND",		0x0077},
				    {"ASR",	"INDEXED_8_X",		0x0067},
				    {"ASR",	"INDEXED_8_Y",		0x1867},
				    {"ASRA",	"INHERANT",		0x0047},
				    {"ASRB",	"INHERANT",		0x0057},

				    {"BCC",	"RELATIF",		0x0024},

				    {"BCLR",	"BITSC",		0x0015},
				    {"BCLRX",	"BITSC",		0x001D},
				    {"BCLRY",	"BITSC",		0x181D},

				    {"BCS",	"RELATIF",		0x0025},
				    {"BEQ",	"RELATIF",		0x0027},
				    {"BGE",	"RELATIF",		0x002C},
				    {"BGT",	"RELATIF",		0x002E},
				    {"BHI",	"RELATIF",		0x0022},
				    {"BHS",	"RELATIF",		0x0024},

				    {"BITA",	"IMMEDIAT8",		0x0085},
				    {"BITA",	"DIRECT",		0x0095},
				    {"BITA",	"EXTEND",		0x00B5},
				    {"BITA",	"INDEXED_8_X",		0x00A5},
				    {"BITA",	"INDEXED_8_Y",		0x18A5},

				    {"BITB",	"IMMEDIAT8",		0x00C5},
				    {"BITB",	"DIRECT",		0x00D5},
				    {"BITB",	"EXTEND",		0x00F5},
				    {"BITB",	"INDEXED_8_X",		0x00E5},
				    {"BITB",	"INDEXED_8_Y",		0x18E5},

				    {"BLE",	"RELATIF",		0x002F},
				    {"BLO",	"RELATIF",		0x0025},
				    {"BLS",	"RELATIF",		0x0023},
				    {"BLT",	"RELATIF",		0x002D},
				    {"BMI",	"RELATIF",		0x002B},
				    {"BNE",	"RELATIF",		0x0026},
				    {"BPL",	"RELATIF",		0x002A},
				    {"BRA",	"RELATIF",		0x0020},

				    {"BRCLR",	"BITTB",		0x0013},
				    {"BRCLRX",	"BITTB",		0x001F},
				    {"BRCLRY",	"BITTB",		0x181F},

				    {"BRN",	"RELATIF",		0x0021},

				    {"BRSET",	"BITTB",		0x0012},
				    {"BRSETX",	"BITTB",		0x001E},
				    {"BRSETY",	"BITTB",		0x181E},

				    {"BSET",	"BITSC",		0x0014},
				    {"BSETX",	"BITSC",		0x001C},
				    {"BSETY",	"BITSC",		0x181C},

				    {"BSR",	"RELATIF",		0x008D},
				    {"BVC",	"RELATIF",		0x0028},
				    {"BVS",	"RELATIF",		0x0029},

				    {"CBA",	"INHERANT",		0x0011},
				    {"CLC",	"INHERANT",		0x000C},
				    {"CLI",	"INHERANT",		0x000E},

				    {"CLR",	"EXTEND",		0x007F},
				    {"CLR",	"INDEXED_8_X",		0x006F},
				    {"CLR",	"INDEXED_8_Y",		0x186F},

				    {"CLRA",	"INHERANT",		0x004F},
				    {"CLRB",	"INHERANT",		0x005F},
				    {"CLV",	"INHERANT",		0x000A},

				    {"CMPA",	"IMMEDIAT8",		0x0081},
				    {"CMPA",	"DIRECT",		0x0091},
				    {"CMPA",	"EXTEND",		0x00B1},
				    {"CMPA",	"INDEXED_8_X",		0x00A1},
				    {"CMPA",	"INDEXED_8_Y",		0x18A1},

				    {"CMPB",	"IMMEDIAT8",		0x00C1},
				    {"CMPB",	"DIRECT",		0x00D1},
				    {"CMPB",	"EXTEND",		0x00F1},
				    {"CMPB",	"INDEXED_8_X",		0x00E1},
				    {"CMPB",	"INDEXED_8_Y",		0x18E1},

				    {"COM",	"EXTEND",		0x0073},
				    {"COM",	"INDEXED_8_X",		0x0063},
				    {"COM",	"INDEXED_8_Y",		0x1863},

				    {"COMA",	"INHERANT",		0x0043},
				    {"COMB",	"INHERANT",		0x0053},


				    {"CPD", 	"IMMEDIAT16",		0x1A83},
				    {"CPD", 	"DIRECT",		0x1A93},
				    {"CPD", 	"EXTEND",		0x1AB3},
				    {"CPD",	"INDEXED_8_X",		0x1AA3},
				    {"CPD",	"INDEXED_8_Y",		0xCDA3},

				    {"CPX", 	"IMMEDIAT16",		0x008C},
				    {"CPX", 	"DIRECT",		0x009C},
				    {"CPX", 	"EXTEND",		0x00BC},
				    {"CPX",	"INDEXED_8_X",		0x00AC},
				    {"CPX",	"INDEXED_8_Y",		0xCDAC},

				    {"CPY", 	"IMMEDIAT16",		0x188C},
				    {"CPY", 	"DIRECT",		0x189C},
				    {"CPY", 	"EXTEND",		0x18BC},
				    {"CPY",	"INDEXED_8_X",		0x1AAC},
				    {"CPY",	"INDEXED_8_Y",		0x18AC},

				    {"DAA", 	"INHERANT",		0x0019},

				    {"DEC", 	"EXTEND",		0x007A},
				    {"DEC",	"INDEXED_8_X",		0x006A},
				    {"DEC",	"INDEXED_8_Y",		0x186A},
				    
				    {"DECA", 	"INHERANT",		0x004A},
				    {"DECB", 	"INHERANT",		0x005A},
				    {"DES", 	"INHERANT",		0x0034},
				    {"DEX", 	"INHERANT",		0x0009},
				    {"DEY", 	"INHERANT",		0x1809},
				    
				    {"EORA", 	"IMMEDIAT8",		0x0088},
				    {"EORA", 	"DIRECT",		0x0098},
				    {"EORA", 	"EXTEND",		0x00B8},
				    {"EORA",	"INDEXED_8_X",		0x00A8},
				    {"EORA",	"INDEXED_8_Y",		0x18A8},

				    {"EORB", 	"IMMEDIAT8",		0x00C8},
				    {"EORB", 	"DIRECT",		0x00D8},
				    {"EORB", 	"EXTEND",		0x00F8},
				    {"EORB",	"INDEXED_8_X",		0x00E8},
				    {"EORB",	"INDEXED_8_Y",		0x18E8},

				    {"FDIV", 	"INHERANT",		0x0003},
				    {"IDIV", 	"INHERANT",		0x0002},

				    {"INC", 	"EXTEND",		0x007C},
				    {"INC",	"INDEXED_8_X",		0x006C},
				    {"INC",	"INDEXED_8_Y",		0x186C},

				    {"INCA", 	"INHERANT",		0x004C},
				    {"INCB", 	"INHERANT",		0x005C},

				    {"INS", 	"INHERANT",		0x0031},
				    {"INX", 	"INHERANT",		0x0008},
				    {"INY", 	"INHERANT",		0x1808},

				    {"JMP", 	"EXTEND",		0x007E},
				    {"JMP",	"INDEXED_8_X",		0x006E},
				    {"JMP",	"INDEXED_8_Y",		0x186E},

				    {"JSR", 	"DIRECT",		0x009D},
				    {"JSR", 	"EXTEND",		0x00BD},
				    {"JSR",	"INDEXED_8_X",		0x00AD},
				    {"JSR",	"INDEXED_8_Y",		0x18AD},

				    {"LDAA", 	"IMMEDIAT8",		0x0086},
				    {"LDAA", 	"DIRECT",		0x0096},
				    {"LDAA", 	"EXTEND",		0x00B6},
				    {"LDAA",	"INDEXED_8_X",		0x00A6},
				    {"LDAA",	"INDEXED_8_Y",		0x18A6},

				    {"LDAB", 	"IMMEDIAT8",		0x00C6},
				    {"LDAB", 	"DIRECT",		0x00D6},
				    {"LDAB", 	"EXTEND",		0x00F6},
				    {"LDAB",	"INDEXED_8_X",		0x00E6},
				    {"LDAB",	"INDEXED_8_Y",		0x18E6},

				    {"LDD", 	"IMMEDIAT16",		0x00CC},
				    {"LDD", 	"DIRECT",		0x00DC},
				    {"LDD", 	"EXTEND",		0x00FC},
				    {"LDD",	"INDEXED_8_X",		0x00EC},
				    {"LDD",	"INDEXED_8_Y",		0x18EC},

				    {"LDS", 	"IMMEDIAT16",		0x008E},
				    {"LDS", 	"DIRECT",		0x009E},
				    {"LDS", 	"EXTEND",		0x00BE},
				    {"LDS",	"INDEXED_8_X",		0x00AE},
				    {"LDS",	"INDEXED_8_Y",		0x18AE},

				    {"LDX", 	"IMMEDIAT16",		0x00CE},
				    {"LDX", 	"DIRECT",		0x00DE},
				    {"LDX", 	"EXTEND",		0x00FE},
				    {"LDX",	"INDEXED_8_X",		0x00EE},
				    {"LDX",	"INDEXED_8_Y",		0x18EE},

				    {"LDY", 	"IMMEDIAT16",		0x18CE},
				    {"LDY", 	"DIRECT",		0x18DE},
				    {"LDY", 	"EXTEND",		0x18FE},
				    {"LDY",	"INDEXED_8_X",		0x1AEE},
				    {"LDY",	"INDEXED_8_Y",		0x18EE},

				    {"LSL", 	"EXTEND",		0x0078},
				    {"LSL",	"INDEXED_8_X",		0x0068},
				    {"LSL",	"INDEXED_8_Y",		0x1868},

				    {"LSLA", 	"INHERANT",		0x0048},
				    {"LSLB", 	"INHERANT",		0x0058},
				    {"LSLD", 	"INHERANT",		0x0005},

				    {"LSR", 	"EXTEND",		0x0074},
				    {"LSR",	"INDEXED_8_X",		0x0064},
				    {"LSR",	"INDEXED_8_Y",		0x1864},

				    {"LSRA", 	"INHERANT",		0x0044},
				    {"LSRB", 	"INHERANT",		0x0054},
				    {"LSRD", 	"INHERANT",		0x0004},

				    {"MUL", 	"INHERANT",		0x003D},

				    {"NEG", 	"EXTEND",		0x0070},
				    {"NEG",	"INDEXED_8_X",		0x0060},
				    {"NEG",	"INDEXED_8_Y",		0x1860},

				    {"NEGA", 	"INHERANT",		0x0040},
				    {"NEGB", 	"INHERANT",		0x0050},

				    {"NOP", 	"INHERANT",		0x0001},

				    {"ORAA", 	"IMMEDIAT8",		0x008A},
				    {"ORAA", 	"DIRECT",		0x009A},
				    {"ORAA", 	"EXTEND",		0x00BA},
				    {"ORAA",	"INDEXED_8_X",		0x00AA},
				    {"ORAA",	"INDEXED_8_Y",		0x18AA},

				    {"ORAB", 	"IMMEDIAT8",		0x00CA},
				    {"ORAB", 	"DIRECT",		0x00DA},
				    {"ORAB", 	"EXTEND",		0x00FA},
				    {"ORAB",	"INDEXED_8_X",		0x00EA},
				    {"ORAB",	"INDEXED_8_Y",		0x18EA},

				    {"PSHA", 	"INHERANT",		0x0036},
				    {"PSHB", 	"INHERANT",		0x0037},
				    {"PSHX", 	"INHERANT",		0x003C},
				    {"PSHY", 	"INHERANT",		0x183C},
				    {"PULA", 	"INHERANT",		0x0032},
				    {"PULB", 	"INHERANT",		0x0033},
				    {"PULX", 	"INHERANT",		0x0038},
				    {"PULY", 	"INHERANT",		0x1838},

				    {"ROL", 	"EXTEND",		0x0079},
				    {"ROL",	"INDEXED_8_X",		0x0069},
				    {"ROL",	"INDEXED_8_Y",		0x1869},

				    {"ROLA", 	"INHERANT",		0x0049},
				    {"ROLB", 	"INHERANT",		0x0059},

				    {"ROR", 	"EXTEND",		0x0076},
				    {"ROR",	"INDEXED_8_X",		0x0066},
				    {"ROR",	"INDEXED_8_Y",		0x1866},

				    {"RORA", 	"INHERANT",		0x0046},
				    {"RORB", 	"INHERANT",		0x0056},

				    {"RTI", 	"INHERANT",		0x003B},
				    {"RTS", 	"INHERANT",		0x0039},
				    {"SBA", 	"INHERANT",		0x0010},

				    {"SBCA", 	"IMMEDIAT8",		0x0082},
				    {"SBCA", 	"DIRECT",		0x0092},
				    {"SBCA", 	"EXTEND",		0x00B2},
				    {"SBCA",	"INDEXED_8_X",		0x00A2},
				    {"SBCA",	"INDEXED_8_Y",		0x18A2},

				    {"SBCB", 	"IMMEDIAT8",		0x00C2},
				    {"SBCB", 	"DIRECT",		0x00D2},
				    {"SBCB", 	"EXTEND",		0x00F2},
				    {"SBCB",	"INDEXED_8_X",		0x00E2},
				    {"SBCB",	"INDEXED_8_Y",		0x18E2},

				    {"SEC", 	"INHERANT",		0x000D},
				    {"SEI", 	"INHERANT",		0x000F},
				    {"SEV", 	"INHERANT",		0x000B},

				    {"STAA", 	"DIRECT",		0x0097},
				    {"STAA", 	"EXTEND",		0x00B7},
				    {"STAA",	"INDEXED_8_X",		0x00A7},
				    {"STAA",	"INDEXED_8_Y",		0x18A7},

				    {"STAB", 	"DIRECT",		0x00D7},
				    {"STAB", 	"EXTEND",		0x00F7},
				    {"STAB",	"INDEXED_8_X",		0x00E7},
				    {"STAB",	"INDEXED_8_Y",		0x18E7},

				    {"STAD", 	"DIRECT",		0x00DD},
				    {"STAD", 	"EXTEND",		0x00FD},
				    {"STAD",	"INDEXED_8_X",		0x00ED},
				    {"STAD",	"INDEXED_8_Y",		0x18ED},

				    {"STOP", 	"INHERANT",		0x00CF},

				    {"STS", 	"DIRECT",		0x009F},
				    {"STS", 	"EXTEND",		0x00BF},
				    {"STS",	"INDEXED_8_X",		0x00AF},
				    {"STS",	"INDEXED_8_Y",		0x18AF},

				    {"STX", 	"DIRECT",		0x00DF},
				    {"STX", 	"EXTEND",		0x00FF},
				    {"STX",	"INDEXED_8_X",		0x00EF},
				    {"STX",	"INDEXED_8_Y",		0xCDEF},

				    {"STY", 	"DIRECT",		0x18DF},
				    {"STY", 	"EXTEND",		0x18FF},
				    {"STY",	"INDEXED_8_X",		0x1AEF},
				    {"STY",	"INDEXED_8_Y",		0x18EF},

				    {"SUBA", 	"IMMEDIAT8",		0x0080},
				    {"SUBA", 	"DIRECT",		0x0090},
				    {"SUBA", 	"EXTEND",		0x00B0},
				    {"SUBA",	"INDEXED_8_X",		0x00A0},
				    {"SUBA",	"INDEXED_8_Y",		0x18A0},

				    {"SUBB", 	"IMMEDIAT8",		0x00C0},
				    {"SUBB", 	"DIRECT",		0x00D0},
				    {"SUBB", 	"EXTEND",		0x00F0},
				    {"SUBB",	"INDEXED_8_X",		0x00E0},
				    {"SUBB",	"INDEXED_8_Y",		0x18E0},

				    {"SUBD", 	"IMMEDIAT16",		0x0083},
				    {"SUBD", 	"DIRECT",		0x0093},
				    {"SUBD", 	"EXTEND",		0x00B3},
				    {"SUBD",	"INDEXED_8_X",		0x00A3},
				    {"SUBD",	"INDEXED_8_Y",		0x18A3},

				    {"SWI", 	"INHERANT",		0x003F},
				    {"TAB", 	"INHERANT",		0x0016},
				    {"TAP", 	"INHERANT",		0x0006},
				    {"TBA", 	"INHERANT",		0x0017},
				    {"TEST", 	"INHERANT",		0x0000},
				    {"TPA", 	"INHERANT",		0x0007},

				    {"TST", 	"EXTEND",		0x007D},
				    {"TST",	"INDEXED_8_X",		0x006D},
				    {"TST",	"INDEXED_8_Y",		0x186D},

				    {"TSTA", 	"INHERANT",		0x004D},
				    {"TSTB", 	"INHERANT",		0x005D},
				    {"TSX", 	"INHERANT",		0x0030},
				    {"TSY", 	"INHERANT",		0x1830},
				    {"TXS", 	"INHERANT",		0x0035},
				    {"TYS", 	"INHERANT",		0x1835},
				    {"WAI", 	"INHERANT",		0x003E},
				    {"XGDX", 	"INHERANT",		0x008F},
				    {"XGDY", 	"INHERANT",		0x188F},

				    {"","",0x00	} };	/* MARQUE DE FIN */

#endif

#endif
