*******************************************************************************************
*
*		Echo program version 1 for the 68HC11. 
*		Utilise des offsets par rapport a une adresse fixe de base des registres d'E/S.
*
*		Sources file for CAM assembler.
*
*		FAVARD Laurent, 1996 ATARI Computers range
*
*	File	:	ECHO_X.S
*	Target	:	MC 68HC11F1
*	Date	:	13 November 1996
*	Release	:	30 April 1997
*	Author	:	FAVARD Laurent, Funship
*
*******************************************************************************************

				OPT		MC68HC11	;	Target processor

*
*	Base and relative address registers for HC11
*

REGISTERS		Equ		$1000						;	Base address register (could be changed)

BAUD			Equ		$002B
SCCR1			Equ		$002C
SCCR2			Equ		$002D
SCSR			Equ		$002E
SCDR			Equ		$002F


*
*	Program's constants
*

SCI_BPS19200	Equ		%00100000
SCI_BPS4800		Equ		%00100010	

SCI_FORMAT		Equ		%00000000	;	8 Bits, NO WAKE UP, 1 stop bit
SCI_CONTROL		Equ		%00001100	;	No Interrupt, TxD and RxD Enable

TDRE			Equ		%10000000	;	Transmit Data Register Empty
TC				Equ		%01000000
RDRF			Equ		%00100000	;	Receive Data Register Full

CCR_BITZ		Equ		%11111011	;	Bit Z du CCR

END_STRING		Equ		$00		;	End of string to send
INTERROGATION	Equ		$3F

*******************************************************************************************
*			Start of text code
*******************************************************************************************


				TEXT		
				ORG		$0000		;	Start at the bottom of RAM memory

Main:			lds		#stack		;	Initialise stack pointer
				bsr		InitSCI		;	Initialise SCI

				bsr		Help
				
Echo:			bsr		WaitByteSCI	;	Get a byte from serial port
				cmpa	#INTERROGATION
				bne		Suite
	
				bsr		Help	
				bra		Echo
					
Suite:			bsr		SendByteSCI	;	And send it
				bra		Echo		;	loop

*******************************************************************************************
*	void Help(void)
*
*	Retourne les informations sur le programme HC11
*******************************************************************************************

Help:			ldx		#Chaine
				bsr		SendStringSCI
;				ldx		#CRLF
;				bsr		SendStringSCI
				ldx		#Version
;				bsr		SendStringSCI
;				ldx		#CRLF
				bsr		SendStringSCI

				rts
			
*******************************************************************************************
*	void InitSCI(void)
*
*	SCI initialisation.
*******************************************************************************************


InitSCI:		pshx
				ldx			#REGISTERS

				ldaa      #SCI_CONTROL	;	Initialise SCI port
				staa      SCCR2,X
				ldaa      #SCI_FORMAT	;	Set SCI data format
				staa      SCCR1,X
				ldaa      #SCI_BPS4800	;	Set SCI rate
				staa      BAUD,X

				pulx
				rts

*******************************************************************************************
*	void	SendByte(Accumulator A)
*
*	Send the contents of A accumulator to the SCI port.
*******************************************************************************************

 
SendByteSCI:	pshb
				pshx

				ldx	      #REGISTERS

wait:  			ldab      SCSR,X
         		bitb      #TC		;	Awaiting Data Register empty
          		beq       wait
          		staa	  SCDR,X

				pulx
          		pulb
          		rts


*******************************************************************************************
*	void	SendString(X = @ chaine)
*
*	Send a string to the SCI until the END_STRING mark has been found.
*******************************************************************************************

 
SendStringSCI:	psha

SendSCI:		ldaa      0,X          	;	Load byte to send	
				cmpa      #END_STRING	;	End of string ?
				beq       SendStringEnd	;	Yes, so exit
				bsr       SendByteSCI	;	
				inx
				bra       SendSCI

SendStringEnd:	pula
				rts

*******************************************************************************************
* 	A <= GetByteSCI(void)
*
* 	Look if a byte was received but don't await it.(if Z=0 then Byte received)
*******************************************************************************************


GetByteSCI:		pshb
				pshx

				ldx		  #REGISTERS

				ldab      SCSR,X		;	Status register
				bitb      #RDRF			;	byte received ?
				beq       EndGet		;	No

				ldaa      SCDR,X							;	A <= byte
				psha				
				tpa											;	A <= CCR
				anda      #CCR_BITZ							;	Bit Z <= 0
				tap											;	CCR <= A
				pula

EndGet:			
				pulx
				pulb
				rts

*******************************************************************************************
* 	A <= WaitByteSCI(void)
*
*	Wait a byte from the SCO port.	
*******************************************************************************************

WaitByteSCI:	bsr       GetByteSCI
				beq       WaitByteSCI
				rts

		
*******************************************************************************************
*			Start of data area
*******************************************************************************************


				DATA

Chaine:			dc.b		"MC68HC11F1 is ready - (c) 1997"
				dc.b		13,10,0
Version:		dc.b		"Echo 2.0 indexed X"
				dc.b		13,10,0
;CRLF:			dc.b		13,10,0

*	program 's stack location of 50 bytes

				ORG			$0200
			
;Fond:			ds.b		19
Stack:			dc.b		0		;	Top of internal SRAM

	
				End


